// -*- c++ -*-
/*
  Copyright 2003 John Knottenbelt
  
  This program is free software; you can redistribute it and/or modify
  it under the terms of the GNU General Public License as published by
  the Free Software Foundation; either version 2 of the License, or
  (at your option) any later version.
 
  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.
 
  You should have received a copy of the GNU General Public License
  along with this program; if not, write to the Free Software
  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111, USA.
*/

#if !defined __SCHEDULER_H
#define __SCHEDULER_H

#include <string>
#include <list>
#include "recording.h"
#include "select.h"
#include "dvbd.h"

class ConnectionManager;
class TunerManager;

class Scheduler : public Selectable {
public:
  Scheduler(const std::string &dbtime, TunerManager *tm, ConnectionManager *cm);
  virtual ~Scheduler();

  void add( Recording &r );
  bool remove(int job);

  virtual void addSelectFDs(Select &) const;
  virtual bool isReady(const Select &) const;

  void process();

  void checkOverlaps(StringList &interruptions) const;
  void getSchedule(StringList &s) const;

  bool getNextJobTime( time_t &time ) const;

private:
  typedef std::list<Recording> List;

  void startRecording(const Recording &);

  bool timeUntilNextRecording( long &tv_sec, long &tv_usec ) const;
  bool overlaps(const List &l, const Recording &r) const;
  void checkOverlappingBlock(const List &l, StringList &interruptions) const;

  void writeDB() const;
  void readDB();

  int lastJob;
  List recordings;
  TunerManager *tm;
  ConnectionManager *cm;
  std::string dbfile;
};

#endif // __SCHEDULER_H
